#ifndef BB_H
#define BB_H

/*******************************************************************
 * Includes
 *******************************************************************/
#include "bb_types.h"

/*******************************************************************
 * Defines
 *******************************************************************/
// BlackBox Implementation / Specification Version
#define BLACKBOX_CHIF_SERVICE   0x11
#define BLACKBOX_VERSION         (0x01)

// API return codes
#define BLACKBOX_OK              (0)
#define BLACKBOX_API_ERROR       (1)
#define BLACKBOX_GENERIC_ERROR  (-1)
// Detailed Error Codes
#define BLACKBOX_ERR_ARGS         (2)
#define BLACKBOX_ERR_FULL         (3)
#define BLACKBOX_ERR_MALLOC       (4)
#define BLACKBOX_ERR_HANDLE       (5)
#define BLACKBOX_ERR_BAD_SIZE     (6)
#define BLACKBOX_ERR_NO_FILTER    (7)
#define BLACKBOX_ERR_INVALID_PARM (8)

// Size #defines
#define BLACKBOX_MAX_FEEDERS      (128)
#define BLACKBOX_MAX_BLOB_SIZE   (962)
#define BLACKBOX_MAX_NAME          (32)
#define BLACKBOX_HASH_SIZE       (sizeof(unsigned int))

// Internal structures (used as input to _register routines)
// Status / Flags (_register)
#define BB_STATUS_IN_USE       (0x80000000)
#define BB_STATUS_T0_LOGGED    (0x40000000)
#define BB_STATUS_ARRAY        (0x08000000)
#define BB_STATUS_HASH         (0x04000000)
#define BB_STATUS_NO_COMPARE   (0x02000000)
#define BB_STATUS_OS_PIECE     (0x00800000)
#define BB_STATUS_STATIC       (0x00000010)   /* Same as BB_FLAGS_STAIC below! */
#define BB_STATUS_TIMESTAMP    (0x00000020)   /* Same as BB_FLAGS_STAIC below! */
// Lowest byte reserved - do not use (ORed with FLAGS on occasion)
// Internal ONLY


// Masks used internally for validating the handle/tag
#define BLACKBOX_TAG_MASK      (0xFFFF0000)
#define BLACKBOX_HANDLE_MASK   (0x0000FFFF)

// Filter (F_TYPE)
#define BLACKBOX_FILTER_NO_FILTER  (0)
#define BLACKBOX_FILTER_BYTE       (1)
#define BLACKBOX_FILTER_WORD       (2)
#define BLACKBOX_FILTER_DWORD      (4)

#define BLACKBOX_FILTER_SET      (0x01)

//
// FLAGS FIELD
// 
// External #defines
// FLAGS field
// Some of these apply to data and some to descriptors, and some to both!
#define BB_FLAGS_DESCRIPTOR             (0x80)    
// If DESCRIPTOR
#define BB_FLAGS_DESC_40                (0x40)    
#define BB_FLAGS_DESC_20                (0x20)
#define BB_FLAGS_DESC_10                (0x10)
// Enumeration
#define BB_FLAGS_DESC_CLASS             (0x08)
#define BB_FLAGS_DESC_CODE              (0x04)
#define BB_FLAGS_DESC_FIELD             (0x0C)
// reserved
#define BB_FLAGS_DESC_R_02              (0x02)        // avail
#define BB_FLAGS_DESC_R_01              (0x01)
//#define BB_FLAGS_RESERVED_BELOW             (0x01)
// If DATA
#define BB_FLAGS_DESCRIPTOR                  (0x80)    
#define BB_FLAGS_INSTANCE                    (0x40)    
#define BB_FLAGS_TICK_PRESENT                (0x20)
#define BB_FLAGS_STATIC                      (0x10)
#define BB_FLAGS_T0                          (0x08)
#define BB_FLAGS_OFFSET_PRESENT              (0x04)
#define BB_FLAGS_RESERVED                    (0x02)
#define BB_FLAGS_RESERVED_FUTURE_EXPANSION   (0x01)


// DESCRIPTOR: Severity + Type
// Type - bits 7-4
#define BB_TYP_BINARY           (0)
#define BB_TYP_TEXT             (1 << 4)    
#define BB_TYP_NUM_UNSIGNED_LSB (2 << 4)    
#define BB_TYP_NUM_SIGNED_LSB   (3 << 4)    
#define BB_TYP_NUM_UNSIGNED_MSB (4 << 4)    
#define BB_TYP_NUM_USIGNED_MSB  (5 << 4)    
#define BB_TYP_SIGNAL_BITS      (6 << 4)    
// Default
#define BB_TYP_NUMERIC          (BB_TYP_NUM_UNSIGNED_LSB)

// Severity - bits 3-0
#define BB_SEV_RSVD          (0)    
#define BB_SEV_CRIT          (1)    
#define BB_SEV_NONCRIT       (2)    
#define BB_SEV_INFORM        (3)    
#define BB_SEV_PERIOD        (4)    
#define BB_SEV_CONFIG        (5)    
#define BB_SEV_ERROR         (1)
#define BB_SEV_WARN          (2)
#define BB_SEV_CHANGE        (3)    
// todo?
//#define BB_TYP_INSTANCE      (1 << 7)    

// -------------------------
// Numeric Size 8-15 / Text/blob Length
// Enum in lower bits
#define BB_BINARY_DATA      (0)
#define BB_SIZE_BYTE        (1)    
#define BB_SIZE_WORD        (2)    
#define BB_SIZE_DWORD       (4)    
#define BB_SIZE_QWORD       (8)    
// High bits
// not sued//#define BB_SIZE_BITS        (0x80)    

// -------------------------
// Visibility 16-23
#define BB_VIS_RESERVED     (0x00)    
#define BB_VIS_CUSTOMER     (0x01)    
#define BB_VIS_SERVICE      (0x02)    
#define BB_VIS_ENGINEERING  (0x03)    

// -------------------------
// Reserved 34-31
// Reserved

// Code bits
//#define BB_CODE_DESCRIPTOR  (0x80000000)
//#define BB_CODE_T0          (0x40000000)


#define BB_DESCRIBE_CLASS    (1)
#define BB_DESCRIBE_CODE     (2)
#define BB_DESCRIBE_FIELD    (3)




//
/*******************************************************************
 * Defines
 *******************************************************************/
#define BB_NUMBER_DESC_STRINGS_TRACKED  (32)

/*******************************************************************
 * Structures (for internal management)
 *
 *******************************************************************/
#pragma pack(1)
// Filter - bits used to identify BYTE,WORD,DWORD that are checked/ignored.
typedef struct {
    UINT32 mask;
} BLACKBOX_FILTER;

#pragma pack(1)
/*******************************************************************
 * Data Structures (into BlackBox storage area)
 *
 *******************************************************************/
// DESCRIPTOR
typedef struct {
    UINT8 flags;
    UINT8 classs;
    UINT8 code;
    UINT8 field;
    UINT8 severity_type;
    UINT8 format;
    UINT8 visibility;
    UINT8 rsvd;
    char  desc[88];
} BLACKBOX_DESCRIPTOR_RECORD;

// DATA - formats
typedef struct {
    UINT8  flags;
    UINT8  classs;
    UINT8  code;
    UINT16 length;             // Length of data field
    //UINT8  data;             // Data is variable in length, depending upon length field
} BLACKBOX_DATA_RECORD_SIMPLE;

typedef struct {
    UINT8  flags;
    UINT8  classs;
    UINT8  code;
    UINT8  instance;
    UINT16 length;             // Length of data field
    //UINT8  data;             // Data is variable in length, depending upon length field
} BLACKBOX_DATA_RECORD_WITH_INSTANCE;

typedef struct {
    UINT8  flags;
    UINT8  classs;
    UINT8  code;
    UINT8  field;
    UINT16 length;             // Length of data field
    //UINT8  data;             // Data is variable in length, depending upon length field
} BLACKBOX_DATA_RECORD_WITH_FIELD;

typedef struct {
    UINT8  flags;
    UINT8  classs;
    UINT8  code;
    UINT8  field;
    UINT8  instance;
    UINT16 length;             // Length of data field
    //UINT8  data;             // Data is variable in length, depending upon length field
} BLACKBOX_DATA_RECORD_WITH_FIELD_AND_INSTANCE;

//
// DATA union
//
typedef union {
    BLACKBOX_DATA_RECORD_SIMPLE          simple;
    BLACKBOX_DATA_RECORD_WITH_INSTANCE   instance;
    BLACKBOX_DATA_RECORD_WITH_FIELD      field;
    BLACKBOX_DATA_RECORD_WITH_FIELD_AND_INSTANCE fi;
} BLACKBOX_DATA_RECORD;


// TODO: TIMETICK - still need to add support
typedef struct {
    UINT32 tick;
} BLACKBOX_TICK;


// BLACKBOX
// - These enumerations are important as they come from the CHIF world too - don't change them!
typedef enum
{ 
    // blackbox
    BB_REGISTER = 1,
    BB_UNREGISTER,
    BB_DETAILS,
    BB_DESCRIPTOR,
    BB_FILTER,
    BB_LOG_BLOB,
    BB_LOG_ARRAY,
    BB_LOG_FIELD,
    BB_LOG_INSTANCE,
    BB_LOG_RAW,
    BB_LOG_STATIC,
    BB_COMMAND,
    BB_OS_ALLOCATE
} BB_BLACKBOX_TYPES;

/*******************************************************************
 * MESSAGE STRUCTURES
 *  
 * Request and Response structures passed across the VAS boundary
 * 
 * NOTE:
 * NOTE: Also passed via CHIF, so once decided, we can't change them!
 * NOTE:
 *  
 *******************************************************************/
#define BB_DATA_BUFFER_SIZE    (928)
//#define BB_DATA_BUFFER_SIZE    (432)

/*******************************************************************
 * BlackBox Structures
 *******************************************************************/
#pragma pack(1)
//
// Blackbox
//
typedef struct {
    char name[32];
    unsigned int  classs;
    unsigned int  flags;
    unsigned int  size;
    unsigned int  count;
} BB_REGISTER_MSG;

typedef struct {
    int  handle;
} BB_UNREGISTER_MSG;

typedef struct {
    int  handle;
    unsigned int modify_mask;
    unsigned int classs;
    unsigned int code;
    unsigned int flags;
} BB_DETAIL_MSG;

// BB_DETAIL_MSG.ModifyMask

enum BB_DETAIL_MODIFY_MASK
{
    BB_MODIFY_CLASS         = 0x01,
    BB_MODIFY_CODE          = 0x02,
    BB_MODIFY_FLAGS         = 0x04
};


typedef struct {
    int  handle;
    unsigned int  selection;
    unsigned int  size;
    BLACKBOX_DESCRIPTOR_RECORD toc;    // Interpreted as an array
} BB_DESCRIBE_MSG;

typedef struct {
    int    handle;
    int    type;
    unsigned int    bytes;
    UINT32 filter;
} BB_FILTER_MSG;

typedef struct {
    int  handle;
    unsigned int size;
    unsigned int instance;
    unsigned int field;
    char data[BLACKBOX_MAX_BLOB_SIZE];
} BB_LOG_MSG;

// ALLOC
typedef struct {
    int  classs;
    char name[32];
} BB_ALLOC_MSG;

// ALL
typedef struct {
    int  subtype;
    union {
        BB_REGISTER_MSG    registration;
        BB_UNREGISTER_MSG  unregister;
        BB_DETAIL_MSG      detail;
        BB_DESCRIBE_MSG    describe;
        BB_FILTER_MSG      filter;
        BB_LOG_MSG         log;
        BB_ALLOC_MSG       alloc;
    } cmd;
} BB_BLACKBOX_MESSAGE;

// BLACKBOX
typedef struct {
    int  handle;
} BB_RESPONSE_DATA;


/*******************************************************************
 * MESSAGE STRUCTURES
 *  
 * Request and Response structures passed across the VAS boundary
 *
 *******************************************************************/
typedef struct
{
    int type;
    union {
        BB_BLACKBOX_MESSAGE     bb;
    } bb_msg;
} BB_MESSAGE;

//#pragma pack(4)

typedef struct
{
    int              rcode;
    union {
        BB_RESPONSE_DATA         returned;
    } bb_msg;
} BB_ANSWER;


/*******************************************************************
 * Package Header
 *  
 * This header is populated and pre-fixed to each file as it is
 * prepared for extraction as a package.
 * 
 * Refer to "Excavator Specification' 
 *  
 *******************************************************************/
#define BB_PACKAGE_MAGIC      ("PKG_")
#define BB_PACKAGE_VERSION    (0x0100)

#define BB_PKG_SOURCE_RSVD       (0x0000)
#define BB_PKG_SOURCE_EXCAVATOR  (0x0001)
#define BB_PKG_SOURCE_ILO_HTTP   (0x0002)
#define BB_PKG_SOURCE_ILO_RIBCL  (0x1003)

typedef struct {
    UINT32 signature;             // magic: PKG_
    UINT16 version;            // 0x0100
    UINT16 source;             // see above
    UINT32 filelength;         // length excluding header
    UINT32 reserved_field;
    UINT32 options;             //   
    char   name[64]; 
    UINT32 reserved[7];
    UINT32 checksum;            // adler-32 for validating header
} BB_PACKAGE_HEADER;

typedef struct _pkt_hdr {
    uint16_t pktsz;
    uint16_t seq;
    uint16_t cmd;
    uint16_t svc;
} pkt_hdr;

typedef struct _bb_chif_req {
    pkt_hdr hdr;
    BB_MESSAGE msg;
} bb_chif_req;

typedef struct _bb_chif_resp {
    pkt_hdr hdr;
    BB_ANSWER msg;
} bb_chif_resp;

#pragma pack()

/*******************************************************************
 * Externs
 *******************************************************************/

/*******************************************************************
 * Function Prototypes
 *******************************************************************/
//
#ifdef __cplusplus
extern "C" {
#endif

int bb_class_allocate( const char* name );

int bb_register( const char * name, 
                 unsigned int cls, 
                 unsigned int flags, 
                 unsigned int size, 
                 int * handle );

int bb_register_array( const char * name, 
                 unsigned int cls, 
                 unsigned int flags, 
                 unsigned int size, 
                 unsigned int count, 
                 int * handle );

int bb_unregister( int handle );

int bb_descriptor_class( int handle, 
                 unsigned int size, 
                 const BLACKBOX_DESCRIPTOR_RECORD * toc );

int bb_descriptor_code( int handle, 
                 unsigned int size,  
                 const BLACKBOX_DESCRIPTOR_RECORD * toc );

int bb_descriptor_field( int handle, 
                 unsigned int size, 
                 const BLACKBOX_DESCRIPTOR_RECORD * toc );

int bb_code_set(  int handle, 
                 unsigned int code );

int bb_build_filter( unsigned int size, 
                 const BLACKBOX_DESCRIPTOR_RECORD * recs, 
                 UINT32 * f );

int bb_filter( int handle, 
                 int type, 
                 unsigned int bytes, 
                 const UINT32 * f );

int bb_log( int handle, 
                 const char * data, 
                 unsigned int size );

int bb_log_array( int handle, 
                 const char * data, 
                 unsigned int size );

int bb_log_instance( int handle, 
                 unsigned int instance, 
                 const char * data, 
                 unsigned int size );

int bb_log_static( int handle, 
                 const char * data, 
                 unsigned int size );

int bb_log_field( int handle, 
                 unsigned int instance, 
                 unsigned int field, 
                 const char * data, 
                 unsigned int size );


#ifdef __cplusplus
   }
#endif

/*******************************************************************
 * End Of File
 *******************************************************************/
#endif

