/////////////////////////////////////////////////////////////////////
//
//  bb_ams.h
//
//  Black Box Items for the Agentless Monitoring Service
//
//  Copyright 2011 by Hewlett-Packard Development Company, L.P.
//
/////////////////////////////////////////////////////////////////////

#ifndef INCL_BLACKBOX_AMS_ITEMS_H
#define INCL_BLACKBOX_AMS_ITEMS_H

/////////////////////////////////////////////////////////////////////
//
//  Data is logged to Black Box in variable length binary records.
//  Each data source defines its own record formats.  There is no
//  central authority maintaining a unified specification or header
//  files (see also: chaos).
//
//  AMS records have a fixed length base structure, followed by zero
//  or more variable length NUL terminated strings.
//
/////////////////////////////////////////////////////////////////////

extern int s_ams_bb_class;
extern int s_ams_bb_handle;
#include "bb_types.h"

#pragma pack(1)

/////////////////////////////////////////////////////////////////////
//
//  AMS class
//
/////////////////////////////////////////////////////////////////////

#define BB_CLASS_AMS    "AMS"

/////////////////////////////////////////////////////////////////////
//
//  AMS codes
//
/////////////////////////////////////////////////////////////////////

enum BB_AMS_Codes
{
    BB_CODE_AMS_OS_BOOT             = 1,
    BB_CODE_AMS_OS_SHUTDOWN         = 2,
    BB_CODE_AMS_OS_INFORMATION      = 3,
    BB_CODE_AMS_HOST_NAME           = 4,
    BB_CODE_AMS_DRIVER              = 5,
    BB_CODE_AMS_SERVICE             = 6,
    BB_CODE_AMS_SOFTWARE            = 7,
    BB_CODE_AMS_OS_CRASH            = 8,
    BB_CODE_AMS_OS_EVENT            = 9,
    BB_CODE_AMS_DEVICE_EVENT        = 10,
    BB_CODE_AMS_OS_USAGE            = 11,
    BB_CODE_AMS_MEMORY_USAGE        = 12,
    BB_CODE_AMS_PROCESSOR_USAGE     = 13,
    BB_CODE_AMS_NIC_INFO            = 14,
    BB_CODE_AMS_NIC_LINK            = 15
};

#define BB_CODE_AMS_OS_BOOT_STR         "OS Boot"
#define BB_CODE_AMS_OS_SHUTDOWN_STR     "OS Shutdown"
#define BB_CODE_AMS_OS_INFORMATION_STR  "OS Info"
#define BB_CODE_AMS_HOST_NAME_STR       "Host Name"
#define BB_CODE_AMS_DRIVER_STR          "Driver"
#define BB_CODE_AMS_SERVICE_STR         "Service"
#define BB_CODE_AMS_SOFTWARE_STR        "Packages"
#define BB_CODE_AMS_OS_CRASH_STR        "OS Crash"
#define BB_CODE_AMS_OS_EVENT_STR        "OS Event"
#define BB_CODE_AMS_DEVICE_EVENT_STR    "Device Event"
#define BB_CODE_AMS_OS_USAGE_STR        "OS Usage"
#define BB_CODE_AMS_MEMORY_USAGE_STR    "Memory Usage"
#define BB_CODE_AMS_PROCESSOR_USAGE_STR "Processor Usage"
#define BB_CODE_AMS_NIC_INFO_STR        "NIC Info"
#define BB_CODE_AMS_NIC_LINK_STR        "NIC Link"

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_OS_BOOT
//
//  (There is no data attached to this record)
//
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_OS_SHUTDOWN
//
//  (There is no data attached to this record)
//
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_OS_INFORMATION
//
//  [0] = BB_AMS_OsInfoData
//  [1] = OS name string
//  [2] = OS version string
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_OsInfoData
{
    UINT8   Type;
    UINT8   Vendor;
    UINT16  Bits;           // 32 or 64
    UINT16  Version[4];     // major/minor/build/patch
    UINT16  ServicePack;
};

// BB_AMS_OsInfoData.Type
// (same as SMIF packet 0x0086)

enum BB_AMS_OsType
{
    BB_AMS_OsType_Windows           = 0,
    BB_AMS_OsType_Netware           = 1,
    BB_AMS_OsType_Linux             = 2,
    BB_AMS_OsType_FreeBSD           = 3,
    BB_AMS_OsType_Solaris           = 4,
    BB_AMS_OsType_VMware            = 5,
    BB_AMS_OsType_Other             = 254
};

// BB_AMS_OsInfoData.Vendor
// (same as SMIF packet 0x0086)

enum BB_AMS_OsVendor
{
    BB_AMS_OsVendor_Microsoft       = 0,
    BB_AMS_OsVendor_Novell          = 1,
    BB_AMS_OsVendor_RedHat          = 2,
    BB_AMS_OsVendor_SUSE            = 3,
    BB_AMS_OsVendor_Debian          = 4,
    BB_AMS_OsVendor_VMware          = 5,
    BB_AMS_OsVendor_FreeBSD         = 6,
    BB_AMS_OsVendor_Oracle          = 7,
    BB_AMS_OsVendor_Sun             = BB_AMS_OsVendor_Oracle,
    BB_AMS_OsVendor_Other           = 254
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_HOST_NAME
//
//  [0] = Computer name string
//  [1] = Fully qualified domain name (FQDN) string
//
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_DRIVER
//
//  [0] = BB_AMS_DriverData
//  [1] = Driver name string
//  [2] = Driver filename string
//  [3] = Driver version string
//  [4] = Driver timestamp string
//  [5] = Driver vendor string
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_DriverData
{
    UINT16   Flags;
};

// BB_AMS_OsDriverData.Flags

enum BB_AMS_DriverFlags
{
    BB_AMS_Driver_First             = 0x0001,
    BB_AMS_Driver_Last              = 0x0002,
    BB_AMS_Driver_Load              = 0x0004,
    BB_AMS_Driver_Unload            = 0x0008
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_SERVICE
//
//  [0] = BB_AMS_ServiceData
//  [1] = Service name string
//  [2] = Service filename string
//  [3] = Service version string
//  [4] = Service timestamp string
//  [5] = Service vendor string
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_ServiceData
{
    UINT16   Flags;
};

// BB_AMS_OsDriverData.Flags

enum BB_AMS_ServiceFlags
{
    BB_AMS_Service_First            = BB_AMS_Driver_First,
    BB_AMS_Service_Last             = BB_AMS_Driver_Last,
    BB_AMS_Service_Start            = BB_AMS_Driver_Load,
    BB_AMS_Service_Stop             = BB_AMS_Driver_Unload
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_SOFTWARE
//
//  [0] = BB_AMS_PackageData
//  [1] = Program vendor string
//  [2] = Program name string
//  [3] = Program version string
//  [4] = Installation timestamp string
//  [5] = Installation package name string
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_PackageData
{
    UINT16   Flags;
};

enum BB_AMS_PackageFlags
{
    BB_AMS_Package_First           = 0x0001,
    BB_AMS_Package_Last            = 0x0002,
    BB_AMS_Package_Install         = 0x0004,
    BB_AMS_Package_Uninstall       = 0x0008,
    BB_AMS_Package_HP              = 0x0100,
    BB_AMS_Package_OsServicePack   = 0x0200,
    BB_AMS_Package_OsHotFix        = 0x0400
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_OS_CRASH
//
//  [0] = BB_AMS_OsCrashData
//  [1] = Message text
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_OsCrashData
{
    UINT64  BugCheckCode;
    UINT64  BugCheckData[4];
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_OS_EVENT
//
//  [0] = BB_AMS_OsEventData
//  [1] = Message source
//  [2] = Message text
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_OsEventData
{
    UINT8   Severity;
    UINT8   LogType;
    UINT16  RepeatCount;
    UINT32  Id;
};

// BB_AMS_OsEventData.Severity

enum BB_AMS_EventSeverity
{
    BB_AMS_Severity_Unknown         = 0,
    BB_AMS_Severity_Information     = 1,
    BB_AMS_Severity_Warning         = 2,
    BB_AMS_Severity_Error           = 3
};

// BB_AMS_OsEventData.Log

enum BB_AMS_EventLogType
{
    BB_AMS_LogType_Unknown          = 0,
    BB_AMS_LogType_System           = 1,
    BB_AMS_LogType_Application      = 2
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_DEVICE_EVENT
//
//  [0] = BB_AMS_DeviceEventData
//  [1] = Location string
//  [2] = Device ID string
//  [3] = Device name string
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_DeviceEventData
{
    UINT8   EventType;
};

// BB_AMS_DeviceHotPlugEventData.Operation

enum BB_AMS_DeviceEventType
{
    BB_AMS_DeviceEvent_Insert       = 1,
    BB_AMS_DeviceEvent_Remove       = 2
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_OS_USAGE
//
//  [0] = BB_AMS_OsUsageData
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_OsUsageData
{
    UINT32   ProcessCount;
    UINT32   ThreadCount;
    UINT32   HandleCount;
    UINT32   LoadAverage[3];
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_MEMORY_USAGE
//
//  [0] = BB_AMS_MemoryUsageData
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_MemoryUsageData
{
    UINT64   PhysicalTotal;
    UINT64   PhysicalFree;
    UINT64   VirtualTotal;
    UINT64   VirtualFree;
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_PROCESSOR_USAGE
//
//  [0] = BB_AMS_ProcessorUsageData
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_ProcessorUsageData
{
    UINT8    User;
    UINT8    Kernel;
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_NIC_INFO
//
//  [0] = BB_AMS_NicInfoData
//  [1] = NIC description string
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_NicInfoData
{
    UINT32  Interface;
    UINT16  Flags;
    UINT8   MacAddress[6];
    UINT8   Ip4Address[4];
    UINT8   Ip6Address[16];
};

// BB_AMS_NicInfoData.Flags and BB_AMS_NicLinkData.Flags

enum BB_AMS_NicFlags
{
    BB_AMS_Nic_First                = 0x0001,
    BB_AMS_Nic_Last                 = 0x0002,
    BB_AMS_Nic_Add                  = 0x0004,
    BB_AMS_Nic_Remove               = 0x0008,
    BB_AMS_Nic_LinkState            = 0x0080,
    BB_AMS_Nic_LinkUp               = 0x0080,
    BB_AMS_Nic_LinkDown             = 0x0000
};

/////////////////////////////////////////////////////////////////////
//
//  BB_CODE_AMS_NIC_LINK
//
//  [0] = BB_AMS_NicLinkData
//
/////////////////////////////////////////////////////////////////////

struct BB_AMS_NicLinkData
{
    UINT32  Interface;
    UINT16  Flags;
};

#pragma pack()

typedef struct BB_AMS_OsInfoData            BB_AMS_OsInfoData;
typedef struct BB_AMS_DriverData            BB_AMS_DriverData;
typedef struct BB_AMS_ServiceData           BB_AMS_ServiceData;
typedef struct BB_AMS_PackageData          BB_AMS_PackageData;
typedef struct BB_AMS_OsCrashData           BB_AMS_OsCrashData;
typedef struct BB_AMS_OsEventData           BB_AMS_OsEventData;
typedef struct BB_AMS_DeviceEventData       BB_AMS_DeviceEventData;
typedef struct BB_AMS_OsUsageData           BB_AMS_OsUsageData;
typedef struct BB_AMS_MemoryUsageData       BB_AMS_MemoryUsageData;
typedef struct BB_AMS_ProcessorUsageData    BB_AMS_ProcessorUsageData;
typedef struct BB_AMS_NicInfoData           BB_AMS_NicInfoData;
typedef struct BB_AMS_NicLinkData           BB_AMS_NicLinkData;
typedef enum   BB_AMS_Codes                 BB_AMS_Codes;
typedef enum   BB_AMS_OsType                BB_AMS_OsType;
typedef enum   BB_AMS_OsVendor              BB_AMS_OsVendor;
typedef enum   BB_AMS_DriverFlags           BB_AMS_DriverFlags;
typedef enum   BB_AMS_ServiceFlags          BB_AMS_ServiceFlags;
typedef enum   BB_AMS_PackageFlags         BB_AMS_PackageFlags;
typedef enum   BB_AMS_EventSeverity         BB_AMS_EventSeverity;
typedef enum   BB_AMS_EventLogType          BB_AMS_EventLogType;
typedef enum   BB_AMS_DeviceEventType       BB_AMS_DeviceEventType;
typedef enum   BB_AMS_NicFlags              BB_AMS_NicFlags;

#ifdef __cplusplus
extern "C" 
#endif
void bb_ams_daemon(int* pAMSRunning,int* pBBRunning);

#endif // INCL_BLACKBOX_AMS_ITEMS_H
