#define _BSD_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>

#include "cpqHost/libcpqhost/cpqhost.h"
#include "bb_ams.h"
#include "bb.h"
#include "bb_data.h"
#include "data_access/bb_driver.h"

#include "net-snmp/net-snmp-config.h"
#include "net-snmp/library/snmp_impl.h"
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/output_api.h>
#include "net-snmp/library/tools.h"

int bb_init();
int doCodeDescriptor(unsigned int, unsigned int, unsigned char, unsigned char,
                     unsigned char, unsigned char, unsigned char, char *);

int getDriver();

drv_entry **drivers;

static field fld[]= {
    {BB_SIZE_WORD,   BB_TYP_NUMERIC, "Flags"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "Name"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "Filename"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "Version"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "Timestamp"},
};
void BB_LOG_DRIVER(
        /*BB_AMS_OsType Type,
          BB_AMS_OsVendor Vendor,
          UINT16 Bits,
          const char* NameString,
          const char* VersionString,
          const UINT16 Version[4],
          UINT16 ServicePack*/)
{
    int i = 0;
    int drvcount;
    int rc;
    int num_descript;
    size_t toc_sz = sizeof(BLACKBOX_DESCRIPTOR_RECORD);;
    size_t blob_sz = sizeof(BB_AMS_DriverData);
    size_t string_list_sz = 0;
    BB_AMS_DriverData *DriverData;

    char *blob;
    BLACKBOX_DESCRIPTOR_RECORD *toc;

    if (!bb_init() )
        return;

   /* do Code Descriptor */
    if ((rc = doCodeDescriptor(s_ams_bb_handle,
                               s_ams_bb_class,
                               BB_CODE_AMS_DRIVER,
                               0,
                               BB_SEV_CONFIG,
                               BB_BINARY_DATA,
                               BB_VIS_CUSTOMER,
                               BB_CODE_AMS_DRIVER_STR)) != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:bb_log_driver","Driver register descriptor failed %d\n", rc));
        return;
    }

    /* build descriptor toc */
    num_descript = sizeof(fld)/sizeof(field);
    DEBUGMSGTL(("bb:log","Driver num_descript = %d\n", num_descript));
    if ((toc_sz *= num_descript) > BLACKBOX_MAX_BLOB_SIZE) {
        DEBUGMSGTL(("bb:log","Driver Descriptor too large %ld\n", toc_sz));
        return;
    }
    DEBUGMSGTL(("bb:log","Driver toc_sz = %ld\n", toc_sz));
    if ((toc = malloc(toc_sz)) == NULL) {
        DEBUGMSGTL(("bb:log","Driver Unable to malloc() %ld bytes\n", toc_sz));
        return;
    }

    /* now do the field descriptor */
    memset(toc, 0, toc_sz);
    for ( i = 0; i < num_descript; i++) {
        toc[i].flags = BB_FLAGS_DESCRIPTOR | BB_FLAGS_DESC_FIELD;
        toc[i].classs = s_ams_bb_class;
        toc[i].code = BB_CODE_AMS_DRIVER;
        toc[i].field = i;
        toc[i].severity_type = BB_DESC_SEVERITY_CONFIGURATION | fld[i].tp;
        toc[i].format = fld[i].sz;
        toc[i].visibility =  BB_DESC_VISIBILITY_CUSTOMER;
        strncpy(toc[i].desc, fld[i].nm, strlen(fld[i].nm));
        DEBUGMSGTL(("bb:log", "toc[i] = %p\n", &toc[i]));
        dump_chunk("bb:bb_log", "descriptor", (const u_char *)&toc[i], 96);
    }

    dump_chunk("bb:bb_log", "toc", (const u_char *)toc, toc_sz);

    if ((rc = bb_descriptor_field(s_ams_bb_handle, toc_sz, toc))
            != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log","Driver register descriptor failed %d\n", rc));
        return;
    }
    free(toc);

    /* Let's go ahead and set the code for */
    if ((rc = bb_code_set(s_ams_bb_handle, BB_CODE_AMS_DRIVER))
               != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log", "SetBlackboxFeederCode failed (%d)\n", rc));
        return;
    }

    /* Let's get our blob */
    drvcount = getDriver();
    for (i = 0; i < drvcount; i++ ) {
        char * end;
        string_list_sz = strlen(drivers[i]->name) +
                 strlen(drivers[i]->filename) +
                 strlen(drivers[i]->version) +
                 strlen(drivers[i]->timestamp) +
                 4;
        if ((blob_sz += string_list_sz) > BLACKBOX_MAX_BLOB_SIZE) {
            DEBUGMSGTL(("bb:log","Driver Data too large %ld\n", blob_sz));
            return;
        }
        if ((blob = malloc(blob_sz)) == NULL ) {
            DEBUGMSGTL(("bb:log","Driver Unable to malloc() %ld blob\n", blob_sz));
            goto free;
        }
        memset(blob, 0, blob_sz);

        /* Got the Blob, so let's stuff in the data */
        DriverData = (BB_AMS_DriverData *)blob;
        DriverData->Flags = BB_AMS_Driver_Load;
        if (i == 0)
                DriverData->Flags |= BB_AMS_Driver_First;
        if (i == (drvcount - 1))
                DriverData->Flags |= BB_AMS_Driver_Last;

        end = blob + sizeof(BB_AMS_DriverData);
        strcpy(end, drivers[i]->name);
        end += strlen(drivers[i]->name) + 1;

        strcpy(end, drivers[i]->filename);
        end += strlen(drivers[i]->filename) + 1;

        strcpy(end, drivers[i]->version);
        end += strlen(drivers[i]->version) + 1;

        strcpy(end, drivers[i]->timestamp);

        DEBUGMSGTL(("bb:log","Module name %s, Driver filename  %s\n",
           drivers[i]->name, drivers[i]->filename));
        DEBUGMSGTL(("bb:log","Module version %s, Driver timestamp  %s\n",
           drivers[i]->version, drivers[i]->timestamp));

        // Log the record
        if ((rc = bb_log(s_ams_bb_handle, (const char*)blob, blob_sz)) !=
                            BLACKBOX_OK) {
            DEBUGMSGTL(("bb:log", "LogBlackboxData failed (%d)\n",rc));
        }

        DEBUGMSGTL(("bb:log", "Logged record for code %d\n",
                    BB_CODE_AMS_DRIVER));
free:
        free(blob);
        if (drivers[i]->timestamp != NULL) free(drivers[i]->timestamp);
        if (drivers[i]->version != NULL) free(drivers[i]->version);
        if (drivers[i]->filename != NULL) free(drivers[i]->filename);
        if (drivers[i]->name != NULL) free(drivers[i]->name);
        free(drivers[i]);
    }
    free(drivers);
}

