//#include <net-snmp/net-snmp-config.h>

#include <sys/time.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include "bb.h"
#include "common/utils.h"

#ifdef UTEST
#define snmp_log(level, format ...) fprintf(stderr, format, ## __VA_ARGS)
#define DEBUGMSGTL(level, format, ...) fprintf(stderr, format, ## __VA_ARGS)`
#else
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/output_api.h>
#endif

#define MAXHPILOCHANNELS 24

#define HPILO_CCB "/dev/hpilo/d0ccb"

int bb_fd = -1;
extern int s_ams_bb_class;
extern int s_ams_bb_handle;
extern int proc_usage_reinit;
extern int mem_usage_reinit;
extern int os_usage_reinit;

void close_bb(int reset)
{
    if (bb_fd != -1) {
        DEBUGMSGTL(("bb:arch", "close %d\n", bb_fd));
        close(bb_fd);
        bb_fd = -1;
    }
    if (reset) {
        s_ams_bb_class = -1;
        s_ams_bb_handle = -1;
        proc_usage_reinit = 1;
        mem_usage_reinit = 1;
        os_usage_reinit = 1;
        
    }
}

void init_bb(void)  {

    int  start = -1;
    char dev[256] = "";
    struct stat buf;

    if (bb_fd != -1 )
        return;

    if ((start = get_sysfs_int("/sys/module/hpilo/parameters/max_ccb")) < 0) {
        start = MAXHPILOCHANNELS - 1;
        
        sprintf(dev,"%s%d", HPILO_CCB, start);
        while ((start > 0) &&  (stat(dev, &buf) < 0)) {
            start = start - 8;
            sprintf(dev,"%s%d", HPILO_CCB, start);
        }
    } else
        start--;

    while (start >= 0 && ((bb_fd = open(dev, O_RDWR|O_EXCL)) == -1)) 
        sprintf(dev,"%s%d", HPILO_CCB, --start);

    if (start <  0) {
        snmp_log(LOG_ERR, "Failed: no available HP iLO channels\n");
        bb_fd = -1;
    }  
    DEBUGMSGTL(("bb:arch", "open %d\n", bb_fd));
    return;
}

int doWrite( bb_chif_req * req, int size)
{
    int cnt = -1;

    if (bb_fd == -1)
        init_bb();

    req->hdr.pktsz = size;
    dump_chunk("bb:arch","Write", (const u_char *)req, size);
    if((cnt = write(bb_fd, req, size)) < 0) {
        return -1;
    }
    DEBUGMSGTL(("bb:arch", "write %d bytes\n", cnt));
    return cnt;
}

int doRead( void * resp, int size)
{
    int cnt = -1;

    if (bb_fd == -1)
        init_bb();

    while ((cnt = read(bb_fd, resp, size)) < 0) {
        if (cnt != EINTR) {
	    DEBUGMSGTL(("bb:bb_log_error", "Read error = %d fd = %d\n",errno, bb_fd));
            return -1;
	}
    }
    DEBUGMSGTL(("bb:arch", "read %d bytes fd = %d\n", cnt, bb_fd));
    dump_chunk("bb:arch","Read", (const u_char *)resp, cnt);

    return (cnt);
}

int doWriteRead( void * req, int size, bb_chif_resp * resp)
{
    int cnt = -1;

    if (bb_fd == -1)
        init_bb();

    dump_chunk("bb:arch","WriteRead-write", (const u_char *)req, size);
    if((cnt = write(bb_fd, req, size)) < 0) {
	DEBUGMSGTL(("bb:bb_log_error", "WriteRead write error = %d fd = %d\n",errno, bb_fd));
        close_bb(1);
        return -1;
    }
    DEBUGMSGTL(("bb:arch", "WriteRead write %d bytes fd = %d\n", cnt, bb_fd));

    memset (resp, 0, sizeof(bb_chif_resp));
    while ((cnt = read(bb_fd, resp, sizeof(bb_chif_resp))) < 0) {
        if (cnt != EINTR) {
	    DEBUGMSGTL(("bb:bb_log_error", "WriteRead read error = %d fd = %d\n",errno, bb_fd));
            close_bb(1);
            return -1;
	}
        DEBUGMSGTL(("bb:arch", "WriteRead read %d bytes fd = snmpHPILODomain.csnmpHPILODomain.c%d\n", cnt, bb_fd));
    }
    dump_chunk("bb:arch","WriteRead-read", (const u_char *)resp, cnt);
    DEBUGMSGTL(("bb:arch", "WriteRead-read %d bytes\n", cnt));
    return (resp->msg.rcode);
}

