#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "bb.h"
#include "bb_ams.h"
#include "net-snmp/net-snmp-config.h"
#include "net-snmp/library/snmp_impl.h"
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/output_api.h>
#include "net-snmp/library/tools.h"

#ifndef true
#define true 1
#endif
#ifndef false
#define false 1
#endif

#define BB_BLACKBOX 2
#ifdef UTEST
void dump_chif(bb_chif_req * req) {
    fprintf(stderr, "pktsz = %d ",req->hdr.pktsz);
    fprintf(stderr, "seq = %d   ",req->hdr.seq);
    fprintf(stderr, "cmd = %d   ",req->hdr.cmd);
    fprintf(stderr, "svc = %d\n",req->hdr.svc);
}

void dump_alloc_req(bb_chif_req* req) {
    fprintf(stderr, "CHIF Request: ");
    dump_chif(req);

    fprintf(stderr, "request->type = %d\n",req->msg.type);
    fprintf(stderr, "request->bb_msg.subtype = %d\n",
				req->msg.bb_msg.bb.subtype);
    fprintf(stderr, "request->bb_msg.cmd.alloc.name = %s\n",
				req->msg.bb_msg.bb.cmd.alloc.name);
}

void dump_alloc_resp(bb_chif_resp* resp) {
    fprintf(stderr, "CHIF Response: ");
    dump_chif(resp);

    fprintf(stderr, "response->rcode = %d\n",resp->msg.rcode);
    fprintf(stderr, "response->bb_msg.returned.handle = %d\n",
				resp->msg.bb_msg.returned.handle);
}

#else
void dump_chif(bb_chif_req* req) {
}
void dump_alloc_req(bb_chif_req* resp) {
}
void dump_alloc_resp(bb_chif_resp* resp) {
}

#endif
int doWrite( bb_chif_req *, int);
int doRead( void *, int);
int doWriteRead(void *, int, bb_chif_resp *);

int s_ams_bb_class = -1;
int s_ams_bb_handle = -1;

static uint16_t s_sequence = 0;

/* Only available for OS clients (via CHIF) for OS API callers from _service */
int bb_class_allocate( const char* name )
{
    bb_chif_req req;
    bb_chif_resp resp;
    int  size;
    BB_ALLOC_MSG *alloc_msg; 

    DEBUGMSGTL(("bb:bb_log", "bb_class_allocate\n"));
    memset (&req,0,sizeof(bb_chif_req));
    
    req.hdr.svc = BLACKBOX_CHIF_SERVICE;
    req.hdr.seq = ++s_sequence;

    req.msg.type = BB_BLACKBOX;
    req.msg.bb_msg.bb.subtype = BB_OS_ALLOCATE;
    alloc_msg = &req.msg.bb_msg.bb.cmd.alloc;
    strncpy(alloc_msg->name, name, sizeof(alloc_msg->name) - 1);
    alloc_msg->name[sizeof(alloc_msg->name) - 1] = 0;
	//size = (int)alloc_msg - (int)&req + sizeof(alloc_msg->name) + 4;
	size = sizeof(pkt_hdr) + sizeof(BB_ALLOC_MSG) + 8;
	dump_alloc_req(&req);
    if (doWrite(&req, size) > 0 )
        if (doRead(&resp,sizeof(bb_chif_resp)) > 0) {
	    dump_alloc_resp(&resp);
            return (resp.msg.rcode);
        }
    
    return (-1);
}

/* Register */
int bb_register(const char * name, 
                unsigned int cls, 
                unsigned int flags, 
                unsigned int size, 
                int * handle )
{
    bb_chif_resp resp;
    int rc;
    bb_chif_req req;
    BB_REGISTER_MSG *register_msg;

    DEBUGMSGTL(("bb:bb_log", "bb_register\n"));
    memset (&req,0,sizeof(bb_chif_req));

    req.hdr.svc = BLACKBOX_CHIF_SERVICE;
    req.hdr.seq = ++s_sequence;

    req.msg.type = BB_BLACKBOX;

    req.msg.bb_msg.bb.subtype = BB_REGISTER;

    register_msg = &req.msg.bb_msg.bb.cmd.registration;
    register_msg->classs = cls;
    register_msg->flags = flags;
    register_msg->size = size;
    register_msg->count = 0;

    strncpy(register_msg->name, name, sizeof(register_msg->name) - 1);
    register_msg->name[sizeof(register_msg->name) - 1] = 0;

    if ((rc = doWriteRead(&req, sizeof(BB_REGISTER_MSG) + 16, &resp)) == 0)
        *handle = resp.msg.bb_msg.returned.handle;
    return (rc);
}

int bb_register_array(const char * name, 
                      unsigned int cls, 
                      unsigned int flags, 
                      unsigned int size, 
                      unsigned int count, 
                      int * handle )
{
    bb_chif_resp resp;
    int rc;
    bb_chif_req req;
    BB_REGISTER_MSG *register_msg;

    DEBUGMSGTL(("bb:bb_log", "bb_register_array\n"));
    memset (&req,0,sizeof(bb_chif_req));

    req.hdr.svc = BLACKBOX_CHIF_SERVICE;
    req.hdr.seq = ++s_sequence;

    req.msg.type = BB_BLACKBOX;

    req.msg.bb_msg.bb.subtype = BB_REGISTER;

    register_msg = &req.msg.bb_msg.bb.cmd.registration;
    register_msg->classs = cls;
    register_msg->flags = flags;
    register_msg->size = size;
    register_msg->count = count;

    strncpy(register_msg->name, name, sizeof(register_msg->name) - 1);
    register_msg->name[sizeof(register_msg->name) - 1] = 0;

    if ((rc = doWriteRead(&req, sizeof(BB_REGISTER_MSG) + 16, &resp)) == 0)
        *handle = resp.msg.bb_msg.returned.handle;
    return (rc);
}

int bb_unregister(int handle)
{
    bb_chif_resp resp;
    int rc;
    bb_chif_req req;
    BB_UNREGISTER_MSG *unregister_msg;

    DEBUGMSGTL(("bb:bb_log", "bb_unregister\n"));
    memset (&req,0,sizeof(bb_chif_req));

    req.hdr.svc = BLACKBOX_CHIF_SERVICE;
    req.hdr.seq = ++s_sequence;

    req.msg.type = BB_BLACKBOX;

    req.msg.bb_msg.bb.subtype = BB_REGISTER;

    unregister_msg = &req.msg.bb_msg.bb.cmd.unregister;
    unregister_msg->handle = handle;

    if ((rc = doWriteRead(&req, sizeof(BB_UNREGISTER_MSG) + 16, &resp)) == 0) {
        handle = resp.msg.bb_msg.returned.handle;
        DEBUGMSGTL(("bb:bb_log", "handle = %x\n", handle));
    }
    return (rc);
}

static int _bb_descriptor(int handle, 
                        unsigned int size, 
                        int descriptor_type,
                        const BLACKBOX_DESCRIPTOR_RECORD * toc )
{
    bb_chif_resp resp;
    int rc;
    bb_chif_req req;
    BB_DESCRIBE_MSG *describe_msg;

    DEBUGMSGTL(("bb:bb_log", "_bb_descriptor\n"));
    memset (&req,0,sizeof(bb_chif_req));

    req.hdr.svc = BLACKBOX_CHIF_SERVICE;
    req.hdr.seq = ++s_sequence;

    req.msg.type = BB_BLACKBOX;

    req.msg.bb_msg.bb.subtype = BB_DESCRIPTOR;

    describe_msg = &req.msg.bb_msg.bb.cmd.describe;
    describe_msg->handle = handle;
    describe_msg->size = size;
    describe_msg->selection = descriptor_type;
    memcpy(&describe_msg->toc, toc, size);
    dump_chunk("bb:bb_log","Request Packet", (const u_char *)&req, 
                                             16+size + (3 * sizeof(int)));

    if ((rc = doWriteRead(&req, 16 + size + (3 * sizeof(int)) ,&resp)) == 0) {
        handle = resp.msg.bb_msg.returned.handle;
        DEBUGMSGTL(("bb:bb_log", "handle = %x\n", handle));
    }
    return (rc);
}

int bb_descriptor_class(int handle, 
                        unsigned int size, 
                        const BLACKBOX_DESCRIPTOR_RECORD * toc )
{
    DEBUGMSGTL(("bb:bb_log", "bb_descriptor_class\n"));
    return (_bb_descriptor(handle, size, BB_DESCRIBE_CLASS, toc));
}

int bb_descriptor_code(int handle, 
                        unsigned int size, 
                        const BLACKBOX_DESCRIPTOR_RECORD * toc )
{
    DEBUGMSGTL(("bb:bb_log", "bb_descriptor_code\n"));
    return (_bb_descriptor(handle, size, BB_DESCRIBE_CODE, toc));
}

int bb_descriptor_field(int handle, 
                        unsigned int size, 
                        const BLACKBOX_DESCRIPTOR_RECORD * toc )
{
    DEBUGMSGTL(("bb:bb_log", "bb_descriptor_field\n"));
    return (_bb_descriptor(handle, size, BB_DESCRIBE_FIELD, toc));
}

int bb_code_set(int handle, unsigned int code)
{
    bb_chif_resp resp;
    int rc;
    bb_chif_req req;
    BB_DETAIL_MSG *detail_msg;

    DEBUGMSGTL(("bb:bb_log", "bb_code_set\n"));
    memset (&req,0,sizeof(bb_chif_req));

    req.hdr.svc = BLACKBOX_CHIF_SERVICE;
    req.hdr.seq = ++s_sequence;

    req.msg.type = BB_BLACKBOX;

    req.msg.bb_msg.bb.subtype = BB_DETAILS;

    detail_msg = &req.msg.bb_msg.bb.cmd.detail;
    detail_msg->handle = handle;
    //detail_msg->classs = cls;
    detail_msg->code = code;
    detail_msg->modify_mask = BB_MODIFY_CODE;

    if ((rc = doWriteRead(&req, sizeof(BB_DETAIL_MSG) + 16, &resp)) == 0){
        handle = resp.msg.bb_msg.returned.handle;
        DEBUGMSGTL(("bb:bb_log", "handle = %x\n", handle));
    }
    return (rc);
}

#ifdef notdef
int bb_build_filter(unsigned int size, 
                    const BLACKBOX_DESCRIPTOR_RECORD * recs, 
                    UINT32 * f )
{
}
#endif

int bb_filter(int handle, 
              int type, 
              unsigned int bytes, 
              const UINT32 * f )
{
    bb_chif_resp resp;
    int rc;
    bb_chif_req req;
    BB_FILTER_MSG *filter_msg;

    DEBUGMSGTL(("bb:bb_log", "bb_filter\n"));
    memset (&req,0,sizeof(bb_chif_req));

    req.hdr.svc = BLACKBOX_CHIF_SERVICE;
    req.hdr.seq = ++s_sequence;

    req.msg.type = BB_BLACKBOX;

    req.msg.bb_msg.bb.subtype = BB_FILTER;

    filter_msg = &req.msg.bb_msg.bb.cmd.filter;
    filter_msg->handle = handle;
    filter_msg->bytes = bytes;
    if (bytes != 0) 
        memcpy(&filter_msg->filter,f,bytes*sizeof(UINT32));

    if ((rc = doWriteRead(&req, sizeof(BB_FILTER_MSG) + 16 + bytes, &resp)) == 0) {
        handle = resp.msg.bb_msg.returned.handle;
        DEBUGMSGTL(("bb:bb_log", "handle = %x\n", handle));
    }
    return (rc);
}

static int _bb_log( int handle, 
            int log_type,
            int instance,
            int field,
            const char * data, 
            unsigned int size )
{
    bb_chif_resp resp;
    int rc;
    bb_chif_req req;
    BB_LOG_MSG *log_msg;

    DEBUGMSGTL(("bb:bb_log", "_bb_log\n"));
    memset(&req,  0,sizeof(bb_chif_req));
    memset(&resp, 0, sizeof(bb_chif_resp));

    req.hdr.svc = BLACKBOX_CHIF_SERVICE;
    req.hdr.seq = ++s_sequence;

    req.msg.type = BB_BLACKBOX;

    req.msg.bb_msg.bb.subtype = log_type;

    log_msg = &req.msg.bb_msg.bb.cmd.log;

    log_msg->field = field;
    log_msg->instance = instance;
    log_msg->handle = handle;
    log_msg->size = size;
    if (size != 0) 
        memcpy(log_msg->data, data, size);

    if ((rc = doWriteRead(&req, size + 32, &resp)) == 0) {
        handle = resp.msg.bb_msg.returned.handle;
        DEBUGMSGTL(("bb:bb_log", "handle = %x\n", handle));
    }
    return (rc);
}

int bb_log( int handle, 
            const char * data, 
            unsigned int size )
{
    DEBUGMSGTL(("bb:bb_log", "bb_log\n"));
    return (_bb_log(handle, BB_LOG_BLOB, 0, 0, data, size));
}

int bb_log_array(int handle, 
                 const char * data, 
                 unsigned int size )
{
    DEBUGMSGTL(("bb:bb_log", "bb_log_array\n"));
    return (_bb_log(handle, BB_LOG_ARRAY, 0, 0, data, size));
}

int bb_log_instance(int handle, 
                    unsigned int instance, 
                    const char * data, 
                    unsigned int size )
{
    DEBUGMSGTL(("bb:bb_log", "bb_log_instance\n"));
    return (_bb_log(handle, BB_LOG_INSTANCE, instance, 0, data, size));
}

int bb_log_field(int handle, 
                    unsigned int instance, 
                    unsigned int field, 
                    const char * data, 
                    unsigned int size )
{
    DEBUGMSGTL(("bb:bb_log", "bb_log_field\n"));
    return (_bb_log(handle, BB_LOG_FIELD, instance, field, data, size));
}

int bb_log_static(int handle, 
                  const char * data, 
                  unsigned int size )
{
    DEBUGMSGTL(("bb:bb_log", "bb_log_static\n"));
    return (_bb_log(handle, BB_LOG_STATIC, 0, 0, data, size));
}

int bb_init() {
    int rc;
    if (s_ams_bb_class == -1 ) { 
        if ((s_ams_bb_class = bb_class_allocate(BB_CLASS_AMS)) == -1) {
            DEBUGMSGTL(("bb:bb_log_error", "Can't allocate AMS class\n"));
            return false;
        }
    }
    DEBUGMSGTL(("bb:bb_log", "AMS class = %d\n",s_ams_bb_class));

    if (s_ams_bb_handle == -1 ) {
        if ((rc = bb_register(BB_CLASS_AMS, s_ams_bb_class, 
                  BB_STATUS_NO_COMPARE, BLACKBOX_MAX_BLOB_SIZE, 
                  &s_ams_bb_handle)) != BLACKBOX_OK ) {
            DEBUGMSGTL(("bb:bb_log_error", "RegisterBlackboxFeeder failed (%d)\n", 
                            rc));
            return false;
        }
    }
    DEBUGMSGTL(("bb:bb_log", "Handle = %x\n", s_ams_bb_handle));
    return true;
}

