#define _BSD_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "cpqHost/libcpqhost/cpqhost.h"
#include "bb_ams.h"
#include "bb.h"
#include "bb_data.h"

#include "net-snmp/net-snmp-config.h"
#include "net-snmp/library/snmp_impl.h"
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/output_api.h>
#include "net-snmp/library/tools.h"

int bb_init();
int doCodeDescriptor(unsigned int, unsigned int, unsigned char, unsigned char,
                     unsigned char, unsigned char, unsigned char, char *);

static field fld[] = {
      {BB_SIZE_BYTE,   BB_TYP_NUMERIC, "Name"},
      {BB_SIZE_BYTE,   BB_TYP_NUMERIC, "Vendor"},
      {BB_SIZE_WORD,   BB_TYP_NUMERIC, "Bits"},
      {BB_SIZE_WORD,   BB_TYP_NUMERIC, "Version-Major"},
      {BB_SIZE_WORD,   BB_TYP_NUMERIC, "Version-Minor"},
      {BB_SIZE_WORD,   BB_TYP_NUMERIC, "Build Number"},
      {BB_SIZE_WORD,   BB_TYP_NUMERIC, "Patch Level"},
      {BB_SIZE_WORD,   BB_TYP_NUMERIC, "Service Pack"},
      {BB_BINARY_DATA, BB_TYP_TEXT,    "OS Version"},
      {BB_BINARY_DATA, BB_TYP_TEXT,    "Kernel Version"},
};

void BB_LOG_OS_INFORMATION()
{
    int i = 0;
    int rc;
    int num_descript;
    size_t toc_sz = sizeof(BLACKBOX_DESCRIPTOR_RECORD);
    size_t blob_sz = sizeof(BB_AMS_OsInfoData);
    size_t string_list_sz = 0;
    BB_AMS_OsInfoData *OsInfoData;
    distro_id_t *myDistro;
    char * end;

    char *blob;
    BLACKBOX_DESCRIPTOR_RECORD *toc;

    if (!bb_init() )
        return;
        
   /* do Code Descriptor */
    if ((rc = doCodeDescriptor(s_ams_bb_handle,
                               s_ams_bb_class,
                               BB_CODE_AMS_OS_INFORMATION,
                               0,
                               BB_SEV_CONFIG,
                               BB_BINARY_DATA,
                               BB_VIS_CUSTOMER,
                               BB_CODE_AMS_OS_INFORMATION_STR)) != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:bb_log_osinfo","OS Information register descriptor failed %d\n", rc));
        return;
    }

    /* build descriptor toc */
    num_descript = sizeof(fld)/sizeof(field);
    DEBUGMSGTL(("bb:log","OS Info num_descript = %d\n", num_descript));
    if ((toc_sz *= num_descript) > BLACKBOX_MAX_BLOB_SIZE) {
        DEBUGMSGTL(("bb:log","OS Info Descriptor too large %ld\n", toc_sz));
        return;
    }
    DEBUGMSGTL(("bb:log","OS Info toc_sz = %ld\n", toc_sz));
    if ((toc = malloc(toc_sz)) == NULL) {
        DEBUGMSGTL(("bb:log","OS Info Unable to malloc() %ld bytes\n", toc_sz));
        return;
    }

    /* now do the field descriptor */
    memset(toc, 0, toc_sz);
    for ( i = 0; i < num_descript; i++) {
        toc[i].flags = BB_FLAGS_DESCRIPTOR | BB_FLAGS_DESC_FIELD;
        toc[i].classs = s_ams_bb_class;
        toc[i].code = BB_CODE_AMS_OS_INFORMATION;
        toc[i].field = i;
        toc[i].severity_type = BB_DESC_SEVERITY_CONFIGURATION | fld[i].tp;
        toc[i].format = fld[i].sz;
        toc[i].visibility =  BB_DESC_VISIBILITY_CUSTOMER;
        strncpy(toc[i].desc, fld[i].nm, strlen(fld[i].nm));
        DEBUGMSGTL(("bb:log", "toc[i] = %p\n", &toc[i]));
        dump_chunk("bb:bb_log","descriptor", (const u_char *)&toc[i], 96);
    }
       
    dump_chunk("bb:bb_log","toc", (const u_char *)toc, toc_sz);

    if ((rc = bb_descriptor_field(s_ams_bb_handle, toc_sz, toc)) 
            != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log","OS Info register descriptor failed %d\n", rc));
        return;
    }
    free(toc);

    /* Let's go ahead and set the code for */
    if ((rc = bb_code_set(s_ams_bb_handle, BB_CODE_AMS_OS_INFORMATION)) 
                != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log", "SetBlackboxFeederCode failed (%d)\n", rc));
        return;
    }

    /* Get the distro Info */
    myDistro = getDistroInfo();

    /* Let's get our blob */
    string_list_sz = strlen((const char *)myDistro->LongDistro) +
                       strlen((const char *)myDistro->KernelVersion) + 3;
    if ((blob_sz += string_list_sz) > BLACKBOX_MAX_BLOB_SIZE) {
        DEBUGMSGTL(("bb:log","OS Info Data too large %ld\n", blob_sz));
        return;
    }
    if ((blob = malloc(blob_sz)) == NULL ) {
        DEBUGMSGTL(("bb:log","OS Info Unable to malloc() %ld blob\n", blob_sz));
        return;
    }
    memset(blob, 0, blob_sz);

    /* Got the Blob, so let's stuff in the data */
    OsInfoData = (BB_AMS_OsInfoData *)blob;
    OsInfoData->Type = BB_AMS_OsType_Linux;

    OsInfoData->Vendor = BB_AMS_OsVendor_Other;
    if (strcmp((const char *)myDistro->Vendor, "Red Hat") == 0)
        OsInfoData->Vendor = BB_AMS_OsVendor_RedHat;
    else
        if (strcmp((const char *)myDistro->Vendor, "SuSE") == 0)
            OsInfoData->Vendor = BB_AMS_OsVendor_SUSE;

    OsInfoData->Bits = myDistro->Bits;
    DEBUGMSGTL(("bb:log","OS Info Type %d, Vendor %d, Bits %d\n",
                OsInfoData->Type, OsInfoData->Vendor, OsInfoData->Bits));

    OsInfoData->Version[0] = myDistro->Version;
    OsInfoData->Version[1] = myDistro->SubVersion;
    OsInfoData->Version[2] = myDistro->BuildNumber;
    OsInfoData->Version[3] = 0;
    DEBUGMSGTL(("bb:log","OS Info "
                "Version[0] %d, Version[1] %d, Version[2] %d, Version[3] %d\n",
                OsInfoData->Version[0], OsInfoData->Version[1], 
                OsInfoData->Version[2], OsInfoData->Version[3]));

    OsInfoData->ServicePack = myDistro->ServicePack;

    end  = blob + sizeof(BB_AMS_OsInfoData);

    strcpy(end, (const char *)myDistro->LongDistro);
    DEBUGMSGTL(("bb:log","OS Info %s, KernelVersion %s\n",
                myDistro->LongDistro, myDistro->LongDistro));

    end += strlen((const char *)myDistro->LongDistro);

    strcpy(end, (const char *)myDistro->KernelVersion);
    DEBUGMSGTL(("bb:log","OS Info %s, KernelVersion %s\n",
                myDistro->LongDistro, myDistro->KernelVersion));

    // Log the record
    if ((rc = bb_log(s_ams_bb_handle, (const char*)blob, blob_sz)) != 
             BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log", "LogBlackboxData failed (%d)\n",rc));
    }

    DEBUGMSGTL(("bb:log", "Logged record for code %d\n", 
                BB_CODE_AMS_OS_INFORMATION));
    free(blob);
    return;

}


