#define _BSD_SOURCE
#include <stdio.h> 
#include <stdlib.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "bb_ams.h"
#include "bb.h"
//#include "bb_data.h"

#include "net-snmp/net-snmp-config.h"
#include "net-snmp/library/snmp_impl.h"
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/output_api.h>
#include "net-snmp/library/snmp_alarm.h"
#include "net-snmp/library/tools.h"

int bb_init();
int doCodeDescriptor(unsigned int, unsigned int, unsigned char, unsigned char,
                     unsigned char, unsigned char, unsigned char, char *);

extern int bb_interval;
extern void bb_log_os_usage(unsigned int, void *);
extern int os_usage_reinit;

static field fld[] = {
      {BB_SIZE_DWORD,   BB_TYP_NUMERIC, "Process Count"},
      {BB_SIZE_DWORD,   BB_TYP_NUMERIC, "Thread Count"},
      {BB_SIZE_DWORD,   BB_TYP_NUMERIC, "Handle Count"},
      {BB_SIZE_DWORD,   BB_TYP_NUMERIC, "Load Avg (1 min)"},
      {BB_SIZE_DWORD,   BB_TYP_NUMERIC, "Load Avg (5 min)"},
      {BB_SIZE_DWORD,   BB_TYP_NUMERIC, "Load Avg (15 min)"},
};

void BB_LOG_OS_USAGE()
{
    int i = 0;
    int rc;
    int num_descript;
    size_t toc_sz = 0;
    BLACKBOX_DESCRIPTOR_RECORD *toc;


    if (!bb_init() )
        return;

   /* do Code Descriptor */
    if ((rc = doCodeDescriptor(s_ams_bb_handle,
                               s_ams_bb_class,
                               BB_CODE_AMS_OS_USAGE,
                               0,
                               BB_SEV_PERIOD,
                               BB_BINARY_DATA,
                               BB_VIS_CUSTOMER,
                               BB_CODE_AMS_OS_USAGE_STR)) != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:bb_log_error","OS Usage register descriptor failed %d\n", rc));
        return;
    }

    /* build descriptor toc */
    num_descript = sizeof(fld)/sizeof(field);
    DEBUGMSGTL(("bb:log","OS Usage num_descript = %d\n", num_descript));
    if ((toc_sz = sizeof(BLACKBOX_DESCRIPTOR_RECORD) * num_descript)
            > BLACKBOX_MAX_BLOB_SIZE) {
        DEBUGMSGTL(("bb:bb_log_error","OS Usage Descriptor too large %ld\n", toc_sz));
        return;
    }
    DEBUGMSGTL(("bb:log","OS Usage toc_sz = %ld\n", toc_sz));
    if ((toc = malloc(toc_sz)) == NULL) {
        DEBUGMSGTL(("bb:log","OS Usage Unable to malloc() %ld bytes\n", toc_sz));
        return;
    }

    /* now do the field descriptor */
    memset(toc, 0, toc_sz);
    for ( i = 0; i < num_descript; i++) {
        toc[i].flags = BB_FLAGS_DESCRIPTOR | BB_FLAGS_DESC_FIELD;
        toc[i].classs = s_ams_bb_class;
        toc[i].code = BB_CODE_AMS_OS_USAGE;
        toc[i].field = i;
        toc[i].severity_type = BB_SEV_PERIOD | fld[i].tp;
        toc[i].format = fld[i].sz;
        toc[i].visibility = BB_VIS_CUSTOMER;
        strncpy(toc[i].desc, fld[i].nm, strlen(fld[i].nm));
        DEBUGMSGTL(("bb:log", "toc[i] = %p\n", &toc[i]));
        dump_chunk("bb:bb_log","descriptor",(const u_char *)&toc[i],96);
    }

    dump_chunk("bb:bb_log","toc", (const u_char *) toc,toc_sz);

    if ((rc = bb_descriptor_field(s_ams_bb_handle, toc_sz, toc))
            != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:bb_log_error","OS Usage register descriptor failed %d\n", rc));
        return;
    }
    free(toc);

    if (!os_usage_reinit) 
        switch (bb_interval) {
            case 0:
                bb_log_os_usage(0, NULL);
                break;
            case 1:
                snmp_alarm_register(60, SA_REPEAT, &bb_log_os_usage, NULL);
                break;
            case 2:
                snmp_alarm_register(600, SA_REPEAT, &bb_log_os_usage, NULL);
                break;
            case 3:
                snmp_alarm_register(3600, SA_REPEAT, &bb_log_os_usage, NULL);
                break;
        }
    else
        os_usage_reinit = 0;
    return;
}

