#define _BSD_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>

#include "cpqHost/libcpqhost/cpqhost.h"
#include "bb_ams.h"
#include "bb.h"
#include "bb_data.h"
#include "data_access/bb_package.h"

#include "net-snmp/net-snmp-config.h"
#include "net-snmp/library/snmp_impl.h"
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/output_api.h>
#include "net-snmp/library/tools.h"

int bb_init();
int doCodeDescriptor(unsigned int, unsigned int, unsigned char, unsigned char,
                     unsigned char, unsigned char, unsigned char, char *);

pkg_entry **packages;

static field fld[]= {
    {BB_SIZE_WORD,   BB_TYP_NUMERIC, "Flags"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "Vendor"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "Name"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "Version"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "Timestamp"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "Package"},
};

int getPackage();
void BB_LOG_PACKAGE(
        /*BB_AMS_OsType Type,
          BB_AMS_OsVendor Vendor,
          UINT16 Bits,
          const char* NameString,
          const char* VersionString,
          const UINT16 Version[4],
          UINT16 PackagePack*/)
{
    int i, j;

    int rc;
    int num_descript;
    size_t toc_sz = 0;
    size_t blob_sz = 5;
    BB_AMS_PackageData *PackageData;

    char *blob;
    BLACKBOX_DESCRIPTOR_RECORD *toc;
    int pkgcount = 0;


    if (!bb_init() )
        return;

   /* do Code Descriptor */
    if ((rc = doCodeDescriptor(s_ams_bb_handle,
                               s_ams_bb_class,
                               BB_CODE_AMS_SOFTWARE,
                               0,
                               BB_SEV_CONFIG,
                               BB_BINARY_DATA,
                               BB_VIS_CUSTOMER,
                               BB_CODE_AMS_SOFTWARE_STR)) != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:bb_log_package","Package register descriptor failed %d\n", rc));
        return;
    }

    /* build descriptor toc */
    num_descript = sizeof(fld)/sizeof(field);
    DEBUGMSGTL(("bb:log","Package num_descript = %d\n", num_descript));
    if ((toc_sz = sizeof(BLACKBOX_DESCRIPTOR_RECORD) * num_descript)
            > BLACKBOX_MAX_BLOB_SIZE) {
        DEBUGMSGTL(("bb:log","Package Descriptor too large %ld\n", toc_sz));
        return;
    }
    DEBUGMSGTL(("bb:log","Package toc_sz = %ld\n", toc_sz));
    if ((toc = malloc(toc_sz)) == NULL) {
        DEBUGMSGTL(("bb:log","Package Unable to malloc() %ld bytes\n", toc_sz));
        return;
    }

    /* now do the field descriptor */
    memset(toc, 0, toc_sz);
    for ( i = 0; i < num_descript; i++) {
        toc[i].flags = BB_FLAGS_DESCRIPTOR | BB_FLAGS_DESC_FIELD;
        toc[i].classs = s_ams_bb_class;
        toc[i].code = BB_CODE_AMS_SOFTWARE;
        toc[i].field = i;
        toc[i].severity_type = BB_DESC_SEVERITY_CONFIGURATION | fld[i].tp;
        toc[i].format = fld[i].sz;
        toc[i].visibility =  BB_DESC_VISIBILITY_CUSTOMER;
        strncpy(toc[i].desc, fld[i].nm, strlen(fld[i].nm));
        DEBUGMSGTL(("bb:log_package", "toc[i] = %p\n", &toc[i]));
        dump_chunk("bb:log_package", "descriptor", (const u_char *)&toc[i], 96);
    }

    dump_chunk("bb:log_package","toc",(const u_char *)toc, toc_sz);

    if ((rc = bb_descriptor_field(s_ams_bb_handle, toc_sz, toc))
            != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log_package", "Package register descriptor failed %d\n", rc));
        return;
    }
    free(toc);

    /* Let's go ahead and set the code for */
    if ((rc = bb_code_set(s_ams_bb_handle, BB_CODE_AMS_SOFTWARE))
               != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log_package", "SetBlackboxFeederCode failed (%d)\n", rc));
        return;
    }

    /* Let's get our blob */
    pkgcount = getPackage();
    DEBUGMSGTL(("bb:log_package","pkgcount = %d\n", pkgcount));
    for (j = 0; j < pkgcount; j++ ) {
        char *end;
        DEBUGMSGTL(("bb:log_package","Iter = %d\n", j));
        blob_sz += sizeof(BB_AMS_PackageData);
	if (packages[j]->vendor != NULL)
            blob_sz += strlen(packages[j]->vendor);
	if (packages[j]->sname != NULL)
            blob_sz += strlen(packages[j]->sname);
	if (packages[j]->version != NULL)
            blob_sz += strlen(packages[j]->version);
	if (packages[j]->name != NULL)
            blob_sz += strlen(packages[j]->name);
	if (packages[j]->timestamp != NULL)
            blob_sz += strlen(packages[j]->timestamp);
        if (blob_sz > BLACKBOX_MAX_BLOB_SIZE) {
            DEBUGMSGTL(("bb:log_package","Package Data too large %ld\n", blob_sz));
            return;
        }
        if ((blob = malloc(blob_sz)) == NULL ) {
            DEBUGMSGTL(("bb:log_package","Package Unable to malloc() %ld blob\n", blob_sz));
            free(packages[j]);
            continue;
        }
        memset(blob, 0, blob_sz);
    
        /* Got the Blob, so let's stuff in the data */
        PackageData = (BB_AMS_PackageData *)blob;
        PackageData->Flags = BB_AMS_Package_Install;
        if (j == 0) 
                PackageData->Flags |= BB_AMS_Package_First;
        if (j == (pkgcount - 1)) 
                PackageData->Flags |= BB_AMS_Package_Last;

        end = blob + sizeof(BB_AMS_PackageData);
	if (packages[j]->vendor != NULL) {
            DEBUGMSGTL(("bb:log:package","Package vendor %s\n", packages[j]->vendor));
            strcpy(end, packages[j]->vendor);
            end += strlen(packages[j]->vendor) + 1;
            free(packages[j]->vendor);
	} else
            end++;

	if (packages[j]->sname != NULL) {
            strcpy(end, packages[j]->sname);
            end += strlen(packages[j]->sname) + 1;
            free(packages[j]->sname);
	} else
            end++;

	if (packages[j]->version != NULL) {
            strcpy(end, packages[j]->version);
            end += strlen(packages[j]->version) + 1;
            free(packages[j]->version);
	} else
            end++;

	if (packages[j]->timestamp != NULL) {
            strcpy(end, packages[j]->timestamp);
            end += strlen(packages[j]->timestamp) + 1;
            free(packages[j]->timestamp);
	} else
            end++;

	if (packages[j]->name != NULL) {
            DEBUGMSGTL(("bb:log:package","Package name %s\n", packages[j]->name));
            strcpy(end, packages[j]->name);
            free(packages[j]->name);
	}


        // Log the record
        if ((rc = bb_log(s_ams_bb_handle, (const char*)blob, blob_sz)) !=
                            BLACKBOX_OK) {
            DEBUGMSGTL(("bb:log", "LogBlackboxData failed (%d)\n",rc));
        }

        DEBUGMSGTL(("bb:log", "Logged record for code %d\n",
                    BB_CODE_AMS_SOFTWARE));
        free(blob);
        free(packages[j]);
    }
    free(packages);
}

