#include <stdio.h> 
#include <stdlib.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>

#include "bb_ams.h"
#include "bb.h"

#include "net-snmp/net-snmp-config.h"
#include "net-snmp/library/snmp_impl.h"
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/output_api.h>
#include "net-snmp/library/snmp_alarm.h"
#include "net-snmp/library/tools.h"

void BB_LOG_PROCESSOR_USAGE();

int bb_init();
int doCodeDescriptor(unsigned int, unsigned int, unsigned char, unsigned char,
                     unsigned char, unsigned char, unsigned char, char *);

extern int file_select(const struct dirent *);
extern int proc_select(const struct dirent *);
extern int bb_interval;
extern void bb_log_proc_usage(unsigned int, void *);
extern int proc_usage_reinit;

static field fld[] = {
      {BB_SIZE_BYTE,   BB_TYP_NUMERIC, "User Time "},
      {BB_SIZE_BYTE,   BB_TYP_NUMERIC, "Kernel Time "},
};

int cpucount;

void BB_LOG_PROCESSOR_USAGE()
{
    int j = 0;
    int rc;
    int num_descript;
    size_t toc_sz = 0;
    size_t max_toc_sz = (10 * sizeof(BLACKBOX_DESCRIPTOR_RECORD));
    struct dirent **cpuent;
    BLACKBOX_DESCRIPTOR_RECORD *toc;
    BLACKBOX_DESCRIPTOR_RECORD *ttoc;

    /* do something different for VmWare and Solaris */
    cpucount = scandir("/sys/class/cpuid", &cpuent, file_select, versionsort);
    DEBUGMSGTL(("bb:bb_log:bb_log_proc_usage","cpucount = %d\n", cpucount));

    if (cpucount == -1) 
        return;

    if (!bb_init() )
        return;

    /* do Code Descriptor */
    if ((rc = doCodeDescriptor(s_ams_bb_handle, 
                               s_ams_bb_class, 
			       BB_CODE_AMS_PROCESSOR_USAGE,
			       0,
			       BB_SEV_PERIOD,
			       BB_BINARY_DATA,
			       BB_VIS_CUSTOMER,
			       BB_CODE_AMS_PROCESSOR_USAGE_STR)) 
            != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:bb_log_error", 
		    "Host Name register descriptor failed %d\n", rc));
        return;
    }

    num_descript = sizeof(fld)/sizeof(field) * cpucount;
    DEBUGMSGTL(("bb:bb_log:bb_log_proc_usage","ProcUsage num_descript = %d\n", 			num_descript));

    toc_sz = sizeof(BLACKBOX_DESCRIPTOR_RECORD) * num_descript;
    toc = malloc(toc_sz);
    DEBUGMSGTL(("bb:log","Initial Processor Usage toc_sz = %ld\n", toc_sz));

    /* now do the field descriptor */
    memset(toc, 0, toc_sz);
    for (j = 0; j < num_descript; j++ ) {
        toc[j].flags = BB_FLAGS_DESCRIPTOR | BB_FLAGS_DESC_FIELD;
        toc[j].classs = s_ams_bb_class;
        toc[j].code = BB_CODE_AMS_PROCESSOR_USAGE;
        toc[j].field = j;
        toc[j].severity_type = BB_SEV_PERIOD | fld[j%2].tp;
        toc[j].format = fld[j%2].sz;
        toc[j].visibility = BB_VIS_CUSTOMER;
        strncpy(toc[j].desc, fld[j%2].nm, strlen(fld[j%2].nm));
        strcat(toc[j].desc, cpuent[j/2]->d_name);
        DEBUGMSGTL(("bb:log", "toc[i] = %p\n", &toc[j]));
        dump_chunk("bb:bb_log","descriptor", (const u_char *)&toc[j], 96);
        if (j % 2)
            free(cpuent[j/2]);
    }
    dump_chunk("bb:bb_log", "toc", (const u_char *)toc, toc_sz);
    free(cpuent);

    ttoc = toc;
    while (toc_sz > BLACKBOX_MAX_BLOB_SIZE) {
        DEBUGMSGTL(("bb:log","Processor Usage ttoc = %p\n", ttoc));
        DEBUGMSGTL(("bb:log","Processor Usage toc_sz = %ld\n", toc_sz));
        if ((rc = bb_descriptor_field(s_ams_bb_handle, 
                                      max_toc_sz,
                                      ttoc)) != BLACKBOX_OK) {
            DEBUGMSGTL(("bb:bb_log_error","Proc Usage register descriptor failed %d\n", 
                        rc));
            return;
        }
        toc_sz = toc_sz - max_toc_sz;
        ttoc = (void *)ttoc + max_toc_sz;
    }
    if ((rc = bb_descriptor_field(s_ams_bb_handle, toc_sz, ttoc)) 
                    != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:bb_log_error","Proc Usage register descriptor failed %d\n", rc));
        return;
    }
    free(toc);

    if (!proc_usage_reinit) 
        switch (bb_interval) {
            case 0:
                bb_log_proc_usage(0, &cpucount);
                break;
            case 1:
                snmp_alarm_register(60, SA_REPEAT, &bb_log_proc_usage, &cpucount);
                break;
            case 2:
                snmp_alarm_register(600, SA_REPEAT, &bb_log_proc_usage, &cpucount);
                break;
            case 3:
                snmp_alarm_register(3600, SA_REPEAT, &bb_log_proc_usage, &cpucount);
                break;
        }
    else
        proc_usage_reinit = 0;
    return;
}

