#define _BSD_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>

#include "cpqHost/libcpqhost/cpqhost.h"
#include "bb_ams.h"
#include "bb.h"
#include "bb_data.h"
#include "data_access/bb_service.h"

#include "net-snmp/net-snmp-config.h"
#include "net-snmp/library/snmp_impl.h"
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/output_api.h>
#include "net-snmp/library/tools.h"

int bb_init();
int doCodeDescriptor(unsigned int, unsigned int, unsigned char, unsigned char,
                     unsigned char, unsigned char, unsigned char, char *);

svc_entry **services;

static field fld[]= {
    {BB_SIZE_WORD,   BB_TYP_NUMERIC, "Flags"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "Name"},
    {BB_BINARY_DATA, BB_TYP_TEXT,    "File"},
};

int getService();
void BB_LOG_SERVICE(
        /*BB_AMS_OsType Type,
          BB_AMS_OsVendor Vendor,
          UINT16 Bits,
          const char* NameString,
          const char* VersionString,
          const UINT16 Version[4],
          UINT16 ServicePack*/)
{
    int i, j;

    int rc;
    int num_descript;
    size_t toc_sz = 0;
    size_t blob_sz = 0;
    BB_AMS_ServiceData *ServiceData;

    char *blob;
    BLACKBOX_DESCRIPTOR_RECORD *toc;
    int svccount = 0;


    if (!bb_init() )
        return;

   /* do Code Descriptor */
    if ((rc = doCodeDescriptor(s_ams_bb_handle,
                               s_ams_bb_class,
                               BB_CODE_AMS_SERVICE,
                               0,
                               BB_SEV_CONFIG,
                               BB_BINARY_DATA,
                               BB_VIS_CUSTOMER,
                               BB_CODE_AMS_SERVICE_STR)) != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:bb_log_service","Service register descriptor failed %d\n", rc));
        return;
    }

    /* build descriptor toc */
    num_descript = sizeof(fld)/sizeof(field);
    DEBUGMSGTL(("bb:log","Service num_descript = %d\n", num_descript));
    if ((toc_sz = sizeof(BLACKBOX_DESCRIPTOR_RECORD) * num_descript)
            > BLACKBOX_MAX_BLOB_SIZE) {
        DEBUGMSGTL(("bb:log","Service Descriptor too large %ld\n", toc_sz));
        return;
    }
    DEBUGMSGTL(("bb:log","Service toc_sz = %ld\n", toc_sz));
    if ((toc = malloc(toc_sz)) == NULL) {
        DEBUGMSGTL(("bb:log","Service Unable to malloc() %ld bytes\n", toc_sz));
        return;
    }

    /* now do the field descriptor */
    memset(toc, 0, toc_sz);
    for ( i = 0; i < num_descript; i++) {
        toc[i].flags = BB_FLAGS_DESCRIPTOR | BB_FLAGS_DESC_FIELD;
        toc[i].classs = s_ams_bb_class;
        toc[i].code = BB_CODE_AMS_SERVICE;
        toc[i].field = i;
        toc[i].severity_type = BB_DESC_SEVERITY_CONFIGURATION | fld[i].tp;
        toc[i].format = fld[i].sz;
        toc[i].visibility =  BB_DESC_VISIBILITY_CUSTOMER;
        strncpy(toc[i].desc, fld[i].nm, strlen(fld[i].nm));
        DEBUGMSGTL(("bb:log_service", "toc[i] = %p\n", &toc[i]));
        dump_chunk("bb:log_service", "descriptor", (const u_char *)&toc[i], 96);
    }

    dump_chunk("bb:log_service","toc",(const u_char *)toc, toc_sz);

    if ((rc = bb_descriptor_field(s_ams_bb_handle, toc_sz, toc))
            != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log_service", "Service register descriptor failed %d\n", rc));
        return;
    }
    free(toc);

    /* Let's go ahead and set the code for */
    if ((rc = bb_code_set(s_ams_bb_handle, BB_CODE_AMS_SERVICE))
               != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log_service", "SetBlackboxFeederCode failed (%d)\n", rc));
        return;
    }

    /* Let's get our blob */
    svccount = getService();
    DEBUGMSGTL(("bb:log_service","svccount = %d\n", svccount));
    for (j = 0; j < svccount; j++ ) {
        char *end;
        DEBUGMSGTL(("bb:log_service","Iter = %d\n", j));
        blob_sz = sizeof(BB_AMS_ServiceData);
        blob_sz += strlen(services[j]->name);
        blob_sz += strlen(services[j]->filename);
        blob_sz += 2;
        if (blob_sz > BLACKBOX_MAX_BLOB_SIZE) {
            DEBUGMSGTL(("bb:log_service","Service Data too large %ld\n", blob_sz));
            return;
        }
        if ((blob = malloc(blob_sz)) == NULL ) {
            DEBUGMSGTL(("bb:log_service","Service Unable to malloc() %ld blob\n", blob_sz));
            free(services[j]);
            continue;
        }
        memset(blob, 0, blob_sz);
    
        /* Got the Blob, so let's stuff in the data */
        ServiceData = (BB_AMS_ServiceData *)blob;
        ServiceData->Flags = BB_AMS_Service_Start;
        if (j == 0) 
                ServiceData->Flags |= BB_AMS_Service_First;
        if (j == (svccount - 1)) 
                ServiceData->Flags |= BB_AMS_Service_Last;

        end = blob + sizeof(BB_AMS_ServiceData);
        strcpy(end, services[j]->name);

        end += strlen(services[j]->name) + 1;
        strcpy(end, services[j]->filename);

        DEBUGMSGTL(("bb:log","Service name %s, Service File Name %s\n",
           services[j]->name, services[j]->filename));

        // Log the record
        if ((rc = bb_log(s_ams_bb_handle, (const char*)blob, blob_sz)) !=
                            BLACKBOX_OK) {
            DEBUGMSGTL(("bb:log", "LogBlackboxData failed (%d)\n",rc));
        }

        DEBUGMSGTL(("bb:log", "Logged record for code %d\n",
                    BB_CODE_AMS_SERVICE));
        free(blob);
        free(services[j]->name);
        free(services[j]->filename);
        free(services[j]);
    }
    free(services);
}

