#define _BSD_SOURCE
#include <stdio.h> 
#include <stdlib.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "bb_ams.h"
#include "bb.h"
#include "bb_data.h"

#include "net-snmp/net-snmp-config.h"
#include "net-snmp/library/snmp_impl.h"
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/output_api.h>

#define netsnmp_cache void
#define SNMP_MAXBUF (4 * 1024)
#include "net-snmp/agent/hardware/memory.h"
#include "net-snmp/agent/hardware/cpu.h"

BB_AMS_MemoryUsageData MemoryUsageData;
int proc_usage_reinit = 0;
int mem_usage_reinit = 0;

extern void close_bb(int);
void bb_log_memory_usage(int regNo, void* unUsed)
{
    int rc;
    netsnmp_memory_info* mem_info;
    size_t blob_sz = sizeof(BB_CODE_AMS_MEMORY_USAGE);

    DEBUGMSGTL(("bbtimer:log", "Memmory Usage Called %d\n", regNo));
    
    if (mem_usage_reinit) 
        BB_LOG_MEMORY_USAGE();

    memset(&MemoryUsageData, 0, blob_sz);

    /* Let's go ahead and set the code for */
    if ((rc = bb_code_set(s_ams_bb_handle, BB_CODE_AMS_MEMORY_USAGE))
                != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log", "SetBlackboxFeederCode failed (%d)\n", rc));
        close_bb(1);
        return;
    }

    netsnmp_memory_load();
    mem_info = netsnmp_memory_get_byIdx(NETSNMP_MEM_TYPE_PHYSMEM, 0);
    if (mem_info) {
        MemoryUsageData.PhysicalTotal = (UINT64)mem_info->size * 
                                        (UINT64)mem_info->units;
        MemoryUsageData.PhysicalFree = (UINT64)mem_info->free * 
                                       (UINT64)mem_info->units;
        //MemoryUsageData.PhysicalTotal = mem_info->size;
        //MemoryUsageData.PhysicalTotal *= mem_info->units;

        //MemoryUsageData.PhysicalFree = mem_info->free;
        //MemoryUsageData.PhysicalFree *= mem_info->units;
    }

    mem_info = netsnmp_memory_get_byIdx(NETSNMP_MEM_TYPE_SWAP, 0);
    if (mem_info) {
        MemoryUsageData.VirtualTotal = (UINT64)mem_info->size * 
                                       (UINT64)mem_info->units;
        MemoryUsageData.VirtualFree = (UINT64)mem_info->free * 
                                      (UINT64)mem_info->units;

        //MemoryUsageData.VirtualFree = mem_info->free;
        //MemoryUsageData.VirtualFree *= mem_info->units;
    }

    // Log the record
    if ((rc = bb_log(s_ams_bb_handle, (const char*)&MemoryUsageData,
                        sizeof(MemoryUsageData))) != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log", "LogBlackboxData failed (%d)\n",rc));
    }

    DEBUGMSGTL(("bb:log", "Logged record for code %d\n",
                BB_CODE_AMS_MEMORY_USAGE));
    close_bb(0);
    return;

}

void bb_log_proc_usage(unsigned int regNo, void *cpucount)
{
    int rc;
    int i;
    BB_AMS_ProcessorUsageData *ProcessorUsageData;
    static void *blob = NULL;
    netsnmp_cpu_info* pInfo;
    size_t blob_sz;
    int *pcount;
    int count;
    pcount  = cpucount;
    count = *pcount;

    DEBUGMSGTL(("bbtimer:log", "Processor Usage Called %d\n", regNo));
    blob_sz = sizeof(BB_AMS_ProcessorUsageData)*count;
    
    if (proc_usage_reinit) 
        BB_LOG_PROCESSOR_USAGE();

    /* Let's go ahead and set the code for */
    if ((rc = bb_code_set(s_ams_bb_handle, BB_CODE_AMS_PROCESSOR_USAGE))
                != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log", "SetBlackboxFeederCode failed (%d)\n", rc));
        close_bb(1);
        return;
    }

    if (blob == NULL) 
        if ((blob = malloc(blob_sz)) == NULL ) {
            DEBUGMSGTL(("bb:log", "Processor Usage Unable to malloc() %ld blob\n", 
                                blob_sz));
            close_bb(1);
            return;
        }
    memset(blob, 0, blob_sz);

    ProcessorUsageData = blob;
    netsnmp_cpu_load();
    pInfo = netsnmp_cpu_get_first();
    for (i = 0; i <  count && pInfo; ++i) {
        ProcessorUsageData->User = 
                    pInfo->total_ticks * 100 / pInfo->user_ticks;
        ProcessorUsageData->Kernel = 
                    pInfo->total_ticks * 100 / pInfo->kern_ticks;
        ProcessorUsageData += sizeof(ProcessorUsageData);
        pInfo = netsnmp_cpu_get_next(pInfo);
    }

    // Log the record
    if ((rc = bb_log(s_ams_bb_handle, (const char*)blob,
                        blob_sz)) != BLACKBOX_OK) {
        DEBUGMSGTL(("bb:log", "LogBlackboxData failed (%d)\n",rc));
    }

    DEBUGMSGTL(("bb:log", "Logged record for code %d\n",
                BB_CODE_AMS_OS_USAGE));
    close_bb(0);
    return;

}
