#ifndef _SNMPHPILODOMAIN_H
#define _SNMPHPILODOMAIN_H

#ifdef NETSNMP_TRANSPORT_HPILO_DOMAIN

#ifdef __cplusplus
extern          "C" {
#endif

#include <net-snmp/library/snmp_transport.h>
#include <net-snmp/library/asn1.h>

/* configurable in driver, default is 8 devices, but hpilo can support up to
 * 32 devices
 */
#define MAXHPILODEVS 24
#define HPILO_CCB "/dev/hpilo/d0ccb"
/*
 * The SNMP over HPILO over IPv4 transport domain is identified by
 * transportDomainStdIpv4 as defined in RFC 3419.
 */

#define TRANSPORT_DOMAIN_HPILO_IP		1,3,6,1,2,1,100,1,232
extern oid netsnmp_snmpHPILODomain[];

    typedef struct netsnmp_ilo_data_s {
       uint16_t from_ilo_seq;
       uint16_t to_ilo_seq;
       int idx;
    } netsnmp_ilo_data;
    
    netsnmp_transport *netsnmp_ilo_transport(const char *instring,
                                             size_t instring_len,
                                             const char *default_target);

    /*
     * "Constructor" for transport domain object.  
     */

    void            netsnmp_hpilo_ctor(void);

#pragma pack(1)

typedef struct {
    uint16_t pktsize;
    uint16_t seqno;
    uint16_t cmdno;
    uint16_t resv;
} pkt_hdr;

#define HELLO_HELLO     1
#define HELLO_GOODBYE   2

/* 0x0093 AgentX Session Setup Request Packet */
typedef struct {
    pkt_hdr  hdr;
    uint16_t op;
    uint16_t flags;
} pkt_0093;

/* 0x8093 AgentX Session Setup Response Packet */
typedef struct {
    pkt_hdr  hdr;
    uint32_t errcode;
} pkt_8093;


/* 0x0094 SNMP AgentX Packet Pass-Through request */
#ifndef AGENTX_PKT_SIZE
#define AGENTX_PKT_SIZE     1472   // From SNMP MAX MESSAGE
#endif


// Packet pkt_0094 and pkt_8094 MUST have buffers aligned and same size.
typedef struct {
    pkt_hdr  hdr;
    uint32_t  pkt_0094_rsvd;   // RESERVED FOR ILO ALIGNMENT
    uint32_t  agentx_req_len;  // Actual size of the data packet
    char    agentx_req[AGENTX_PKT_SIZE];
} pkt_0094;

/* Packet 0x8094: SNMP AgentX Packet Pass-Through response */
#define PKT_8094_ERR_NO_CCB          1  // Need to close device and start over
#define PKT_8094_ERR_NO_SOCKET       2  // No SNMP socket is available.
#define PKT_8094_ERR_SOCKET_ERR      3  // Socket Connection Lost
#define PKT_8094_ERR_INVALID_PKT     4  // Packet not a valid AgentX structure
#define PKT_8094_ERR_FUBAR           5  // We are Fouled Up Beyond All Repair.
// Start Over.

/* errcode > 255:  Error codes from RFC 2257
 **
 ** NOTE:    If "errcode" less than 256, the contents of the agentx_resp
 **          is undefined and is void.  agentx_resp_len should be 0.
 */

typedef struct {
    pkt_hdr      hdr;
    uint32_t errcode;
    uint32_t agentx_resp_len;  // Actual size of the data packet
    char     agentx_resp[AGENTX_PKT_SIZE];
} pkt_8094;

/* 0x0095 iLO Log Event Packet */
typedef struct {
    pkt_hdr  hdr;
    uint16_t size;
    uint16_t code;
    uint32_t flags;
    char     data[0];
} pkt_0095;

/* 0x8095 iLO Log Event Response */
typedef struct {
    pkt_hdr  hdr;
    uint32_t errcode;
    uint32_t eventno;
} pkt_8095;

#pragma pack()


#ifdef __cplusplus
}
#endif
#endif                          /*NETSNMP_TRANSPORT_HPILO_DOMAIN */

#endif/*_SNMPHPILODOMAIN_H*/
